package com.me.pacman.entity;

import com.me.pacman.level.LevelTile;
import com.me.pacman.state.LevelState;

public abstract class MovableEntity extends Entity {

    // speed in tiles / second.
    public float speed;
    public boolean moving;
    public Direction currDirection;
    private float tolerance;

    private Direction nextDirection = null;
    public boolean canMove = true;

    public MovableEntity(LevelState state, float x, float y, float speed, boolean moving, Direction currDirection, float turnTolerance) {
        super(state, x, y);
        this.speed = speed;
        this.moving = moving;
        this.currDirection = currDirection;
        this.tolerance = turnTolerance;
    }

    public void update(float dt) {
        super.update(dt);

        if (!moving) {
            return;
        }

        LevelTile nextTile = null;

        if (nextDirection != null) {
            boolean turned = false;
            switch (nextDirection) {
                case NORTH:
                    nextTile = state.level.getTile(x, y + 1f);
                    if (nextTile == null) {
                        if (currDirection == Direction.SOUTH) {
                            turned = true;
                        }
                    } else if (nextTile.isPassable() && Math.abs(x - ((int) x + 0.5f)) <= tolerance) {
                        x = ((int) x) + 0.5f;
                        turned = true;
                    }
                    break;
                case EAST:
                    nextTile = state.level.getTile(x + 1f, y);
                    if (nextTile == null) {
                        if (currDirection == Direction.WEST) {
                            turned = true;
                        }
                    } else if (nextTile.isPassable() && Math.abs(y - ((int) y + 0.5f)) <= tolerance) {
                        y = ((int) y) + 0.5f;
                        turned = true;
                    }
                    break;
                case SOUTH:
                    nextTile = state.level.getTile(x, y - 1f);
                    if (nextTile == null) {
                        if (currDirection == Direction.NORTH) {
                            turned = true;
                        }
                    } else if (nextTile.isPassable() && Math.abs(x - ((int) x + 0.5f)) <= tolerance) {
                        x = ((int) x) + 0.5f;
                        turned = true;
                    }
                    break;
                case WEST:
                    nextTile = state.level.getTile(x - 1f, y);
                    if (nextTile == null) {
                        if (currDirection == Direction.EAST) {
                            turned = true;
                        }
                    } else if (nextTile.isPassable() && Math.abs(y - ((int) y + 0.5f)) <= tolerance) {
                        y = ((int) y) + 0.5f;
                        turned = true;
                    }
                    break;
            }
            if (turned) {
                currDirection = nextDirection;
                nextDirection = null;
            }
        }

        float dist = speed * dt;

        LevelTile currentTile = state.level.getTile(x, y);
        nextTile = null;

        float new_y = y;
        float new_x = x;

        switch (currDirection) {
            case NORTH:
                if (currentTile == null && y > state.level.height + 1) {
                    new_y = -1f;
                    canMove = true;
                } else {
                    new_y += dist;
                    nextTile = state.level.getTile(new_x, new_y + 0.5f);
                    canMove = nextTile == null || nextTile.isPassable();
                }
                break;
            case EAST:
                if (currentTile == null && x > state.level.width + 1) {
                    new_x = -1f;
                    canMove = true;
                } else {
                    new_x += dist;
                    nextTile = state.level.getTile(new_x + 0.5f, new_y);
                    canMove = nextTile == null || nextTile.isPassable();
                }
                break;
            case SOUTH:
                if (currentTile == null && y < 0) {
                    new_y = state.level.height + 1;
                    canMove = true;
                } else {
                    new_y -= dist;
                    nextTile = state.level.getTile(new_x, new_y - 0.5f);
                    canMove = nextTile == null || nextTile.isPassable();
                }
                break;
            case WEST:
                if (currentTile == null && x < 0) {
                    new_x = state.level.width + 1;
                    canMove = true;
                } else {
                    new_x -= dist;
                    nextTile = state.level.getTile(new_x - 0.5f, new_y);
                    canMove = nextTile == null || nextTile.isPassable();
                }
                break;
        }

        // if move isn't going to collide with wall, move normally.
        // otherwise, trim would-be decimal and center entity on tile
        x = canMove? new_x : ((int) new_x) + 0.5f;
        y = canMove? new_y : ((int) new_y) + 0.5f;
    }

    public void setNextDirection(Direction direction) {
        if (direction != currDirection) {
            nextDirection = direction;
        }
    }

}
